<?php

namespace RBS\RBSPayment\Model;

use Magento\Quote\Api\Data\CartInterface;
use Magento\Sales\Model\Order\Payment\Transaction;
use Magento\Sales\Model\Order;


require_once(__DIR__ . '/Config/include.php');
/**
 * Class RBSPayment
 * @package RBS\RBSPayment\Model
 */
class RBSPayment extends \Magento\Payment\Model\Method\AbstractMethod
{

//    protected $RBS_PROD_URL = RBS_PROD_URL;
//    protected $RBS_TEST_URL = RBS_TEST_URL;

    /**
     * @var bool
     */
    protected $_isInitializeNeeded = true;
    protected $_isGateway = true;
    /**
     * Payment code
     *
     * @var string
     */
    protected $_code = 'rbspayment';

    /**
     * Availability option
     *
     * @var bool
     */
    protected $_isOffline = false;
    /**
     * Sidebar payment info block
     *
     * @var string
     */
    protected $_infoBlockType = 'Magento\Payment\Block\Info\Instructions';

    protected $_encryptor;

    protected $orderFactory;

    protected $urlBuilder;

    protected $_transactionBuilder;

    protected $_logger;

    protected $_customLogger;

    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
        \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
        \Magento\Payment\Helper\Data $paymentData,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Payment\Model\Method\Logger $logger,
//        \RBS\RBSPayment\Logger\Logger $customLogger,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate,
        \Magento\Framework\Encryption\EncryptorInterface $encryptor,
//        \Magento\Framework\UrlInterface $urlBuilder,
        \Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface $builderInterface,
        \Magento\Sales\Model\OrderFactory $orderFactory,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = [])
    {
        $this->orderFactory = $orderFactory;
//        $this->urlBuilder = $urlBuilder;
        $this->_transactionBuilder = $builderInterface;
        $this->_encryptor = $encryptor;

        parent::__construct(
            $context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger,
//            $customLogger,
            $resource,
            $resourceCollection,
            $data);

//        $this->_customLogger = $customLogger;
    }

    /**
     * Получить объект Order по его orderId
     *
     * @param $orderId
     * @return Order
     */
    protected function getOrder($orderId)
    {
        return $this->orderFactory->create()->loadByIncrementId($orderId);
    }


    /**
     * Получить сумму платежа по orderId заказа
     *
     * @param $orderId
     * @return float
     */
    public function getAmount($orderId)
    {
        return $this->getOrder($orderId)->getGrandTotal();
    }


    /**
     * Получить идентификатор клиента по orderId заказа
     *
     * @param $orderId
     * @return int|null
     */
    public function getCustomerId($orderId)
    {
        return $this->getOrder($orderId)->getCustomerId();
    }


    /**
     * Получить код используемой валюты по orderId заказа
     *
     * @param $orderId
     * @return null|string
     */
    public function getCurrencyCode($orderId)
    {
        return $this->getOrder($orderId)->getBaseCurrencyCode();
    }


    /**
     * Set order state and status
     * (Этот метод вызывается при нажатии на кнопку "Place Order")
     *
     * @param string $paymentAction
     * @param \Magento\Framework\DataObject $stateObject
     * @return void
     */
    public function initialize($paymentAction, $stateObject)
    {
        $stateObject->setState(Order::STATE_PENDING_PAYMENT);
        $stateObject->setStatus(Order::STATE_PENDING_PAYMENT);
        $stateObject->setIsNotified(false);
    }


    /**
     * Check whether payment method can be used with selected shipping method
     * (Проверка возможности доставки)
     *
     * @param string $shippingMethod
     * @return bool
     */
    protected function isCarrierAllowed($shippingMethod)
    {
        return strpos($this->getConfigData('allowed_carrier'), $shippingMethod) !== false;
    }


    /**
     * Check whether payment method can be used
     * (Проверка на доступность метода оплаты)
     *
     * @param CartInterface|null $quote
     * @return bool
     */
    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null)
    {
        if ($quote === null) {
            return false;
        }
        return parent::isAvailable($quote) && $this->isCarrierAllowed(
            $quote->getShippingAddress()->getShippingMethod()
        );
    }


    /**
     * Получить адрес платежного шлюза
     *
     * @return string
     */
    public function getGateUrl()
    {
        $action_adr = RBS_PROD_URL;
        // for migrate to new payment gateway
        if (defined('RBS_PROD_URL_ALT') && defined('RBS_PROD_URL_ALT_PREFIX')) {
            if (substr($this->getConfigData("RBS_MERCHANT_LOGIN"), 0, strlen(RBS_PROD_URL_ALT_PREFIX)) == RBS_PROD_URL_ALT_PREFIX) {
                $action_adr = RBS_PROD_URL_ALT;
            } else {
                //disable callbacks for old gateway version
            }
        }

        if ($this->getConfigData('test_mode')) {
            $action_adr = RBS_TEST_URL;
        }

        if ($this->getConfigData('two_stage')) {
            $action_adr .= "registerPreAuth.do";
        } else {
            $action_adr .= "register.do";
        }

        return $action_adr;
    }


    public function getStatusUrl()
    {
        $action_adr = RBS_PROD_URL;
        // for migrate to new payment gateway
        if (defined('RBS_PROD_URL_ALT') && defined('RBS_PROD_URL_ALT_PREFIX')) {
            if (substr($this->getConfigData("RBS_MERCHANT_LOGIN"), 0, strlen(RBS_PROD_URL_ALT_PREFIX)) == RBS_PROD_URL_ALT_PREFIX) {
                $action_adr = RBS_PROD_URL_ALT;
            } else {
                //disable callbacks for old gateway version
            }
        }

        if ($this->getConfigData('test_mode')) {
            $action_adr = RBS_TEST_URL;
        }

        $action_adr .= "getOrderStatusExtended.do";

        return $action_adr;
    }



    /**
     * Получить массив параметров для формы оплаты
     *
     * @param $orderId
     * @return array
     */
    public function getPostData($orderId)
    {
        $postData = array(
            'order_id' => $orderId,
//            'orderNumber' => $orderId . "#" . time(),
//            'userName' => $this->getConfigData("RBS_MERCHANT_LOGIN"),
//            'password' => $this->_encryptor->decrypt($this->getConfigData("RBS_MERCHANT_PASSWORD")),
            'amount' => round(number_format($this->getAmount($orderId), 2, '.', '') * 100),
//            'description' => __("Pay order №") . $orderId,
//            'returnUrl' => $this->urlBuilder->getUrl('rbspayment/url/rbssuccess'),
//            http://magento.onemanteam.ru/rbspayment/url/rbssuccess/?orderId=8ed21017-ce2b-7aed-8ed2-1017000be08e&lang=ru

//            'response_url' => $this->urlBuilder->getUrl('checkout/onepage/success'),
//            'currency' => $this->getCurrencyCode($orderId)

//            'jsonParams' => json_encode(
//                array(
//                    'CMS:' => 'Magento 2.2',
//                    'Module-Version: ' => '1.0.0',
//                )
//            ),
        );

        return $postData;
    }

    /**
     * Проверить данные ответного запроса (Pay URL)
     *
     * @param $response
     * @return bool
     */
    private function checkRBSResponse($response)
    {
//        $this->_customLogger->info("checking parameters");
        foreach (
            [
                "orderId",
            ] as $param) {
            if (!isset($response[$param])) {
//                $this->_customLogger->info("Pay URL: required field \"{$param}\" is missing");
                return false;
            }
        }
        $settings = array(
            'login' => $this->getConfigData("RBS_MERCHANT_LOGIN"),
            'password' => $this->_encryptor->decrypt($this->getConfigData("RBS_MERCHANT_PASSWORD"))
        );

        $validated = $this->isPaymentValid($settings, $response);
        if ($validated === true) {
//            $this->_customLogger->info("Responce - OK");
            return true;
        } else {
//            $this->_customLogger->info($validated);
            return false;
        }

    }

    /**
     * Вызывается при запросе Pay URL со стороны RBS
     * @param $responseData
     * @return Order
     */
    public function processResponse($responseData)
    {

        $debugData = ['response' => $responseData];
//        $this->_customLogger->info("processResponse", $debugData);

        if ($this->checkRBSResponse($responseData)) {

            $action_adr = $this->getStatusUrl();

            $args = array(
                'userName' => $this->getConfigData("RBS_MERCHANT_LOGIN"),
                'password' => $this->_encryptor->decrypt($this->getConfigData("RBS_MERCHANT_PASSWORD")),
                'orderId' => $responseData['orderId'],
            );

            $ch = curl_init();
            curl_setopt_array($ch, array(
                CURLOPT_URL => $action_adr,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_SSL_VERIFYPEER => false,
                CURLOPT_POST => true,
                CURLOPT_POSTFIELDS => http_build_query($args, '', '&')
            ));

            $response = curl_exec($ch);
            curl_close($ch);

            $response = json_decode($response, true);

            list($orderId,) = explode('#', $response['orderNumber']);
            $order = $this->getOrder($orderId);
            if ($order && ($this->_processOrder($order, $response) === true)) {
                // echo __("OK");
                return true;
            } else {
                // echo __("FAIL");
                return false;
            }
        }


    }

    /**
     * Метод вызывается при вызове Pay URL
     *
     * @param Order $order
     * @param mixed $response
     * @return bool
     */
    protected function _processOrder(Order $order, $response)
    {

//        $this->_customLogger->info("_processRbs",
//            [
//                "\$order" => $order,
//                "\$response" => $response
//            ]);
        try {
            if (round($order->getGrandTotal() * 100) != $response["amount"]) {
//                $this->_customLogger->info("_processOrder: amount mismatch, order FAILED");
                return false;
            }
            if ($response["orderStatus"] == '2' || $response["orderStatus"] == '1') {

                //todo EIRAno
//                $this->createTransaction($order, $response);
                $order
                    ->setState($this->getConfigData("order_status"))
                    ->setStatus($order->getConfig()->getStateDefaultStatus($this->getConfigData("order_status")))
                    ->save();
//                $this->_customLogger->info("_processOrder: order state changed: STATE_PROCESSING");
//                $this->_customLogger->info("_processOrder: order data saved, order OK");
                return true;
            } else {
                $order
                    ->setState(Order::STATE_CANCELED)
                    ->setStatus($order->getConfig()->getStateDefaultStatus(Order::STATE_CANCELED))
                    ->save();

//                $this->_customLogger->info("_processOrder: order state not STATE_CANCELED");
//                $this->_customLogger->info("_processOrder: order data saved, order not approved");
                return false;
            }

        } catch (\Exception $e) {
//            $this->_customLogger->info("_processOrder exception", $e->getTrace());
            return false;
        }
    }

    public function isPaymentValid($rbsSettings, $response)
    {




//
//        'userName' => $this->getConfigData("RBS_MERCHANT_LOGIN"),
//        'password' => $this->_encryptor->decrypt($this->getConfigData("RBS_MERCHANT_PASSWORD")),
//

//
//        if ($rbsSettings['merchant_id'] != $response['merchant_id']) {
//            return 'An error has occurred during payment. Merchant data is incorrect.';
//        }
//        if ($response['order_status'] == 'declined') {
//            return 'An error has occurred during payment. Order is declined.';
//        }
//
//        $responseSignature = $response['signature'];
//        if (isset($response['response_signature_string'])) {
//            unset($response['response_signature_string']);
//        }
//        if (isset($response['signature'])) {
//            unset($response['signature']);
//        }
//        if ($this->getSignature($response, $rbsSettings['secret_key']) != $responseSignature) {
//            return 'An error has occurred during payment. Signature is not valid.';
//        }


        return true;
    }

    public function createTransaction($order = null, $paymentData = array())
    {
        try {
            //get payment object from order object
            $payment = $order->getPayment();

            $payment->setLastTransId($paymentData['payment_id']);
            $payment->setTransactionId($paymentData['payment_id']);
            $payment->setAdditionalInformation(
                [\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array)$paymentData]
            );
            $formatedPrice = $order->getBaseCurrency()->formatTxt(
                $order->getGrandTotal()
            );

            $message = __('The authorized amount is %1.', $formatedPrice);
            //get the object of builder class
            $trans = $this->_transactionBuilder;
            $transaction = $trans->setPayment($payment)
                ->setOrder($order)
                ->setTransactionId($paymentData['payment_id'])
                ->setAdditionalInformation(
                    [\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array)$paymentData]
                )
                ->setFailSafe(true)
                //build method creates the transaction and returns the object
                ->build(\Magento\Sales\Model\Order\Payment\Transaction::TYPE_ORDER);

            $payment->addTransactionCommentsToOrder(
                $transaction,
                $message
            );
            $payment->setParentTransactionId(null);
            $payment->save();
            $order->save();

            return $transaction->save()->getTransactionId();
        } catch (\Exception $e) {
//            $this->_customLogger->info("_processOrder exception", $e->getTrace());
        }
    }
}