<?php

namespace RBSPAYMENT\Alfabank\Model;

use Magento\Quote\Api\Data\CartInterface;
use Magento\Sales\Model\Order\Payment\Transaction;
use Magento\Sales\Model\Order;

require_once(__DIR__ . '/Config/include.php');
/**
 * Class Alfabank
 * @package RBSPAYMENT\Alfabank\Model
 */
class Alfabank extends \Magento\Payment\Model\Method\AbstractMethod
{

    /**
     * @var \RBSPAYMENT\Alfabank\Api\OrderRepositoryInterface
     */
    protected $gatewayOrderRepository;

    /**
     * @var \RBSPAYMENT\Alfabank\Api\Data\OrderInterfaceFactory
     */
    protected $gatewayOrderFactory;

    /**
     * @var bool
     */
    protected $_isInitializeNeeded = true;
    protected $_isGateway = true;
    /**
     * Payment code
     *
     * @var string
     */
    protected $_code = 'alfabank';

    /**
     * Availability option
     *
     * @var bool
     */
    protected $_isOffline = false;
    /**
     * Sidebar payment info block
     *
     * @var string
     */
    protected $_infoBlockType = 'Magento\Payment\Block\Info\Instructions';

    protected $_encryptor;

    protected $orderFactory;

    protected $urlBuilder;

    protected $_transactionBuilder;

    protected $_logger;

    protected $_customLogger;

    private $allowCallbacks;

    public $gateway_orderId;

    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
        \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
        \Magento\Payment\Helper\Data $paymentData,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Payment\Model\Method\Logger $logger,
//        \RBSPAYMENT\Alfabank\Logger\Logger $customLogger,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate,
        \Magento\Framework\Encryption\EncryptorInterface $encryptor,
//        \Magento\Framework\UrlInterface $urlBuilder,
        \Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface $builderInterface,
        \Magento\Sales\Model\OrderFactory $orderFactory,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        \RBSPAYMENT\Alfabank\Api\OrderRepositoryInterface $gatewayOrderRepository,
        \RBSPAYMENT\Alfabank\Api\Data\OrderInterfaceFactory $gatewayOrderFactory,

        array $data = [])
    {
        $this->orderFactory = $orderFactory;
//        $this->urlBuilder = $urlBuilder;
        $this->_transactionBuilder = $builderInterface;
        $this->_encryptor = $encryptor;
        $this->gatewayOrderRepository = $gatewayOrderRepository;
        $this->gatewayOrderFactory    = $gatewayOrderFactory;

        parent::__construct(
            $context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger,
//            $customLogger,
            $resource,
            $resourceCollection,
            $data);

//        $this->_customLogger = $customLogger;
    }

    /**
     *
     * @param $orderId
     * @return Order
     */
    protected function getOrder($orderId)
    {
        return $this->orderFactory->create()->loadByIncrementId($orderId);
    }


    /**
     *
     * @param $orderId
     * @return float
     */
    public function getAmount($orderId)
    {
        return $this->getOrder($orderId)->getGrandTotal();
    }


    /**
     *
     * @param $orderId
     * @return int|null
     */
    public function getCustomerId($orderId)
    {
        return $this->getOrder($orderId)->getCustomerId();
    }


    /**
     * Get currency code by orderId
     *
     * @param $orderId
     * @return null|string
     */
    public function getCurrencyCode($orderId)
    {
        return $this->getOrder($orderId)->getBaseCurrencyCode();
    }


    /**
     * Set order state and status
     * (this method calls by pressing button "Place Order")
     *
     * @param string $paymentAction
     * @param \Magento\Framework\DataObject $stateObject
     * @return void
     */
    public function initialize($paymentAction, $stateObject)
    {
        $stateObject->setState(Order::STATE_PENDING_PAYMENT);
        $stateObject->setStatus(Order::STATE_PENDING_PAYMENT);
        $stateObject->setIsNotified(false);
    }


    /**
     * Check whether payment method can be used with selected shipping method
     *
     * @param string $shippingMethod
     * @return bool
     */
    protected function isCarrierAllowed($shippingMethod)
    {
        return (isset($shippingMethod) && strpos($this->getConfigData('allowed_carrier'), $shippingMethod) !== false);
    }


    /**
     * Check whether payment method can be used
     *
     * @param CartInterface|null $quote
     * @return bool
     */
    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null)
    {
        if ($quote === null) {
            return false;
        }
        return parent::isAvailable($quote) && ($this->isCarrierAllowed(
            $quote->getShippingAddress()->getShippingMethod()
        ) || $quote->getIsVirtual());
    }


    /**
     *
     * @return string
     */
    public function getGateUrl($with_action = true)
    {
        $action_adr = RBSPAYMENT_PROD_URL;
        // for migrate to new payment gateway
        if (defined('RBSPAYMENT_PROD_URL_ALT') && defined('RBSPAYMENT_PROD_URL_ALT_PREFIX')) {
            if (substr($this->getConfigData("MERCHANT_LOGIN"), 0, strlen(RBSPAYMENT_PROD_URL_ALT_PREFIX)) == RBSPAYMENT_PROD_URL_ALT_PREFIX) {
                $action_adr = RBSPAYMENT_PROD_URL_ALT;
            } else {
                //disable callbacks for old gateway version

            }
        }

        if ($this->getConfigData('test_mode')) {
            $action_adr = RBSPAYMENT_TEST_URL;
        }

        if ($with_action == false) {
            return $action_adr;
        }

        if ($this->getConfigData('two_stage')) {
            $action_adr .= "registerPreAuth.do";
        } else {
            $action_adr .= "register.do";
        }

        return $action_adr;
    }


    public function getStatusUrl()
    {
        $action_adr = RBSPAYMENT_PROD_URL;
        // for migrate to new payment gateway
        if (defined('RBSPAYMENT_PROD_URL_ALT') && defined('RBSPAYMENT_PROD_URL_ALT_PREFIX')) {
            if (substr($this->getConfigData("MERCHANT_LOGIN"), 0, strlen(RBSPAYMENT_PROD_URL_ALT_PREFIX)) == RBSPAYMENT_PROD_URL_ALT_PREFIX) {
                $action_adr = RBSPAYMENT_PROD_URL_ALT;
            } else {
                //disable callbacks for old gateway version
            }
        }

        if ($this->getConfigData('test_mode')) {
            $action_adr = RBSPAYMENT_TEST_URL;
        }

        $action_adr .= "getOrderStatusExtended.do";

        return $action_adr;
    }


    /**
     * get data array for payment form
     *
     * @param $orderId
     * @return array
     */
    public function getPostData($orderId)
    {
        $postData = array(
            'order_id' => $orderId,
//            'orderNumber' => $orderId . "#" . time(),
//            'userName' => $this->getConfigData("MERCHANT_LOGIN"),
//            'password' => $this->_encryptor->decrypt($this->getConfigData("MERCHANT_PASSWORD")),
            'amount' => round(number_format($this->getAmount($orderId), 2, '.', '') * 100),
//            'description' => __("Pay order №") . $orderId,
//            'returnUrl' => $this->urlBuilder->getUrl(),

//            'response_url' => $this->urlBuilder->getUrl('checkout/onepage/success'),
//            'currency' => $this->getCurrencyCode($orderId)

//            'jsonParams' => json_encode(
//                array(
//                    'CMS:' => 'Magento 2.2',
//                    'Module-Version: ' => '1.0.0',
//                )
//            ),
        );

        return $postData;
    }

    /**
     *
     * @param $response
     * @return bool
     */
    private function checkGatewayResponse($response)
    {
//        $this->_customLogger->info("checking parameters");
//        foreach (
//            [
//                "orderId",
//                "mdOrder",
//            ] as $param) {
//            if (!isset($response[$param])) {
////                $this->_customLogger->info("Pay URL: required field \"{$param}\" is missing");
//                return false;
//            }
//        }
        $settings = array(
            'login' => $this->getConfigData("MERCHANT_LOGIN"),
            'password' => $this->_encryptor->decrypt($this->getConfigData("MERCHANT_PASSWORD"))
        );

        $validated = $this->isPaymentValid($settings, $response);
        if ($validated === true) {
//            $this->_customLogger->info("Responce - OK");
            return true;
        } else {
//            $this->_customLogger->info($validated);
            return false;
        }

    }

    /**
     * @param $responseData
     * @param bool $callback
     * @return Order
     */
    public function processResponse($responseData, $is_callback = false)
    {

//        $debugData = ['response' => $responseData];
//        $this->_customLogger->info("processResponse", $debugData);



        if ($this->checkGatewayResponse($responseData)) {

            $action_adr = $this->getStatusUrl();
            $orderId = $is_callback == true ? $responseData['mdOrder'] : $responseData['orderId']; //bzz
            $this->gateway_orderId = $orderId;

            $args = array(
                'userName' => $this->getConfigData("MERCHANT_LOGIN"),
                'password' => $this->_encryptor->decrypt($this->getConfigData("MERCHANT_PASSWORD")),
                'orderId' => $orderId
            );

            $ch = curl_init();
            curl_setopt_array($ch, array(
                CURLOPT_URL => $action_adr,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_SSL_VERIFYPEER => false,
                CURLOPT_POST => true,
                CURLOPT_POSTFIELDS => http_build_query($args, '', '&')
            ));
            $response = curl_exec($ch);
            curl_close($ch);

            $response = json_decode($response, true);

            list($orderId,) = explode('#', $response['orderNumber']);
            $order = $this->getOrder($orderId);


            if (defined('RBSPAYMENT_ENABLE_CALLBACK') && RBSPAYMENT_ENABLE_CALLBACK == true) {
                if ($is_callback == true) {
                    $updateState = true;
                } else {
                    $updateState = false;
                }
            } else {
                $updateState = true;
            }


            if ($order && ($this->_processOrder($order, $response, $updateState) === true)) {
                // echo __("OK");
                return true;
            } else {
                // echo __("FAIL");
                return false;
            }
        }


    }

    /**
     *
     * @param Order $order
     * @param mixed $response
     * @param bool $updateState
     * @return bool
     */
    protected function _processOrder(Order $order, $response, $updateState = false)
    {

//        $this->_customLogger->info("_processRbs",
//            [
//                "\$order" => $order,
//                "\$response" => $response
//            ]);
        try {
            if (round($order->getGrandTotal() * 100) != $response["amount"]) {
//                $this->_customLogger->info("_processOrder: amount mismatch, order FAILED");
                return false;
            }
            if ($response["orderStatus"] == '2' || $response["orderStatus"] == '1') {

                if ($updateState == true) {
                    //todo idabi
//                    $this->createTransaction($order, $response);

                    $order
                        ->setState($this->getConfigData("order_status"))
                        ->setStatus($order->getConfig()->getStateDefaultStatus($this->getConfigData("order_status")))
                        ->save();
//                    $this->_customLogger->info("_processOrder: order state changed: STATE_PROCESSING");
//                    $this->_customLogger->info("_processOrder: order data saved, order OK");
                }
                $this->_saveGatewayData($order, $response);
                return true;

            } else {

                if ($updateState) {
                    $order
                        ->setState(Order::STATE_CANCELED)
                        ->setStatus($order->getConfig()->getStateDefaultStatus(Order::STATE_CANCELED))
                        ->save();
//                    $this->_customLogger->info("_processOrder: order state not STATE_CANCELED");
//                    $this->_customLogger->info("_processOrder: order data saved, order not approved");
                }
                $this->_saveGatewayData($order, $response);
                return false;

            }

        } catch (\Exception $e) {
//            $this->_customLogger->info("_processOrder exception", $e->getTrace());
            return false;
        }
    }



public function _saveGatewayData ($order, $response) {
    /* @var \RBSPAYMENT\Alfabank\Api\Data\OrderInterface $gatewayOrder */
    $gatewayOrder = $this->gatewayOrderFactory->create();

    $orderAmount = $order->getGrandTotal();
    $statusDeposited = (int)$response['orderStatus'];
    $orderAmountDeposited = $orderAmount;

    $gatewayOrder
        ->setIncrementId($order->getIncrementId())
        ->setGatewayOrderReference($this->gateway_orderId)
        ->setOrderAmount($orderAmount)
        ->setOrderAmountDeposited($orderAmountDeposited)
        ->setStatusDeposited($statusDeposited)
        ->setCreatedAt(date('Y-m-d H:i:s'))
        ->setUpdatedAt(date('Y-m-d H:i:s'));

    $this->gatewayOrderRepository->save($gatewayOrder);
}


    public function isPaymentValid($methodSettings, $response)
    {

//        'userName' => $this->getConfigData("MERCHANT_LOGIN"),
//        'password' => $this->_encryptor->decrypt($this->getConfigData("MERCHANT_PASSWORD")),

//        if ($methodSettings['merchant_id'] != $response['merchant_id']) {
//            return 'An error has occurred during payment. Merchant data is incorrect.';
//        }
//        if ($response['order_status'] == 'declined') {
//            return 'An error has occurred during payment. Order is declined.';
//        }
//
//        $responseSignature = $response['signature'];
//        if (isset($response['response_signature_string'])) {
//            unset($response['response_signature_string']);
//        }
//        if (isset($response['signature'])) {
//            unset($response['signature']);
//        }
//        if ($this->getSignature($response, $methodSettings['secret_key']) != $responseSignature) {
//            return 'An error has occurred during payment. Signature is not valid.';
//        }

        return true;
    }

    public function createTransaction($order = null, $paymentData = array())
    {
        try {
            //get payment object from order object
            $payment = $order->getPayment();

            $payment->setLastTransId($paymentData['payment_id']);
            $payment->setTransactionId($paymentData['payment_id']);
            $payment->setAdditionalInformation(
                [\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array)$paymentData]
            );
            $formatedPrice = $order->getBaseCurrency()->formatTxt(
                $order->getGrandTotal()
            );

            $message = __('The authorized amount is %1.', $formatedPrice);
            //get the object of builder class
            $trans = $this->_transactionBuilder;
            $transaction = $trans->setPayment($payment)
                ->setOrder($order)
                ->setTransactionId($paymentData['payment_id'])
                ->setAdditionalInformation(
                    [\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array)$paymentData]
                )
                ->setFailSafe(true)
                //build method creates the transaction and returns the object
                ->build(\Magento\Sales\Model\Order\Payment\Transaction::TYPE_ORDER);

            $payment->addTransactionCommentsToOrder(
                $transaction,
                $message
            );
            $payment->setParentTransactionId(null);
            $payment->save();
            $order->save();

            return $transaction->save()->getTransactionId();
        } catch (\Exception $e) {
//            $this->_customLogger->info("_processOrder exception", $e->getTrace());
        }
    }
}
